@@ -150,7 +150,7 @@ class RoborockClientV1(RoborockClient, ABC):
     """Roborock client base class for version 1 devices."""
 
     _listeners: dict[str, ListenerModel] = {}
-    _map_response_decoder: Callable[[RoborockMessage], MapResponse] | None = None
+    _map_response_decoder: Callable[[RoborockMessage], MapResponse | None] | None = None
 
     def __init__(self, device_info: DeviceData, security_data: SecurityData | None) -> None:
         """Initializes the Roborock client."""
@@ -439,17 +439,24 @@ class RoborockClientV1(RoborockClient, ABC):
                 elif data.payload and protocol == RoborockMessageProtocol.MAP_RESPONSE:
                     if self._map_response_decoder is not None:
                         map_response = self._map_response_decoder(data)
-                        queue = self._waiting_queue.get(map_response.request_id)
-                        if queue:
-                            queue.set_result(map_response.data)
-                        else:
-                            self._logger.debug(
-                                "Received unsolicited map response for request_id %s", map_response.request_id
-                            )
+                        if map_response is not None:
+                            queue = self._waiting_queue.get(map_response.request_id)
+                            if queue:
+                                queue.set_result(map_response.data)
+                            else:
+                                self._logger.debug(
+                                    "Received unsolicited map response for request_id %s", map_response.request_id
+                                )
+                elif data.protocol == RoborockMessageProtocol.GENERAL_RESPONSE and data.payload is None:
+                    # Api will often send blank messages with matching sequences, we can ignore these.
+                    continue
                 else:
                     queue = self._waiting_queue.get(data.seq)
                     if queue:
-                        queue.set_result(data.payload)
+                        if data.protocol == RoborockMessageProtocol.HELLO_RESPONSE:
+                            queue.set_result(data)
+                        else:
+                            queue.set_result(data.payload)
                     else:
                         self._logger.debug("Received response for unknown request id %s", data.seq)
         except Exception as ex:
