@@ -63,3 +63,11 @@ async def test_execute_scene(mock_rest):
     ud = await api.pass_login("password")
     await api.execute_scene(ud, 123456)
     mock_rest.assert_any_call("https://api-us.roborock.com/user/scene/123456/execute", "post")
+
+
+async def test_code_login_v4_flow(mock_rest) -> None:
+    """Test that we can login with a code and we get back the correct userdata object."""
+    api = RoborockApiClient(username="test_user@gmail.com")
+    await api.request_code_v4()
+    ud = await api.code_login_v4(4123, "US", 1)
+    assert ud == UserData.from_dict(USER_DATA)
